@@ -1,19 +1,25 @@
 #'Determine candidate regions of selection
 #'@description Determine candidate regions of selection.
 #'@param scan a data frame containing scores (output of \code{\link{ihh2ihs}}, \code{\link{ines2rsb}} or \code{\link{ies2xpehh}}).
-#'@param threshold boundary score above which markers are defined as "extreme".
+#'@param threshold a positive numeric value. Scores which are higher are considered extreme.
+#'@param negativeThreshold a negative numeric value. Scores which are below are considered extreme.
 #'@param pval logical. If \code{TRUE} use the (negative log-) p-value instead of the score.
-#'@param ignore_sign logical. If \code{TRUE} (default), take absolute values of score.
+#'@param ignore_sign logical. If \code{TRUE}, take absolute values of score.
 #'@param window_size size of sliding windows. If set to 1, no windows
 #'are constructed and only the individual extremal markers are reported.
-#'@param overlap size of window overlap (default 0, i.e. no overlap).
+#'@param overlap size of window overlap (default 0, i.e. no overlap). Note that if you use this
+#'option together with join_neigbhors=TRUE, candidate regions might get bigger since the markers with
+#'extreme scores re-appear in several windows
 #'@param right logical, indicating if the windows should be closed on the right (and open on the left) or vice versa.
 #'@param min_n_mrk minimum number of markers per window.
 #'@param min_n_extr_mrk minimum number of markers with extreme
 #'value in a window.
 #'@param min_perc_extr_mrk minimum percentage of extremal markers among all markers.
 #'@param join_neighbors logical. If \code{TRUE} (default), merge neighboring windows with
-#'extreme values.
+#'extreme values to a bigger interval.
+#'@param keepNA keep markers with a value of NA, i.e. for which no score could be
+#'calculated (e.g. due to a too small minor allele frequency). This option will affect the calculated
+#'number of markers in a window.
 #'@details There is no generally agreed method how to determine genomic
 #'regions which might have been under recent selection. Since selection tends
 #'to yield clusters of markers with outlier values, a common approach is
@@ -30,12 +36,43 @@
 #'@return A data frame with chromosomal regions, i.e. windows that fulfill
 #'the necessary conditions to qualify as candidate regions under selection.
 #'For each region the overall number of markers, their mean and maximum,
-#'the number of markers with extremal values, their percentage of all markers
+#'the number of markers with extreme values, their percentage of all markers
 #'and their average are reported.
+#'In case only a positive threshold is specified, only positive scores are taken into account for
+#'the calculation of mean and maximum values. Vice versa for only a negative threshold being specified.
+#'In case both thresholds are specified, the absolute scores are used for mean and max.
 #'@seealso \code{\link{calc_region_stats}}
+#'@examples
+#'#toy example of an ihs scan
+#'scan <- data.frame(CHR = "1", POSITION = c(2, 3, 6, 7, 8) * 10000, IHS = c(-4, 0.5, 1, 6, NA))
+#'scan
+#'#candidate regions with default window size
+#'calc_candidate_regions(scan, threshold = 2)
+#'#with smaller window size
+#'calc_candidate_regions(scan, threshold = 2, window_size = 20000)
+#'#add negative threshold
+#'calc_candidate_regions(scan, threshold = 2, negativeThreshold = -2, window_size = 20000)
+#'#ignoring sign yields the same
+#'calc_candidate_regions(scan, threshold = 2, ignore_sign = TRUE, window_size = 20000)
+#'#'#use overlapping windows
+#'calc_candidate_regions(scan, threshold = 2, ignore_sign = TRUE, window_size = 20000,
+#'overlap = 10000)
+#'#do not join windows with extreme values
+#'calc_candidate_regions(scan, threshold = 2, ignore_sign = TRUE, window_size = 20000,
+#'overlap = 10000, join_neighbors = FALSE)
+#'#include windows without extreme values by 'min_n_extr_mrk = 0'
+#'calc_candidate_regions(scan, threshold = 2, ignore_sign = TRUE, window_size = 20000,
+#'overlap = 10000, join_neighbors = FALSE, min_n_extr_mrk = 0)
+#'#include markers without score by 'keepNA = TRUE'
+#'calc_candidate_regions(scan, threshold = 2, ignore_sign = TRUE, window_size = 20000,
+#'overlap = 10000, join_neighbors = FALSE, min_n_extr_mrk = 0, keepNA = TRUE)
+#'#include windows without markers by 'min_n_mrk = 0'
+#'calc_candidate_regions(scan, threshold = 2, ignore_sign = TRUE, window_size = 20000,
+#'overlap = 10000, join_neighbors = FALSE, min_n_mrk = 0, min_n_extr_mrk = 0, keepNA = TRUE)
 #'@export
 calc_candidate_regions <- function(scan,
                                    threshold = NA,
+                                   negativeThreshold = NA,
                                    pval = FALSE,
                                    ignore_sign = FALSE,
                                    window_size = 1000000,
@@ -44,7 +81,8 @@ calc_candidate_regions <- function(scan,
                                    min_n_mrk = 1,
                                    min_n_extr_mrk = 1,
                                    min_perc_extr_mrk = 0,
-                                   join_neighbors = TRUE) {
+                                   join_neighbors = TRUE,
+                                   keepNA = FALSE) {
   ## check parameters
   
   if (!is.null(scan$iHS)) {
@@ -59,10 +97,36 @@ calc_candidate_regions <- function(scan,
   if (!("POSITION" %in% colnames(scan))) {
     stop("Scan does not contain a column named 'POSITION'.", call. = FALSE)
   }
-  if (is.na(threshold) | !is.numeric(threshold)) {
-    stop("Specify a numeric value for 'threshold'.",
+  if (is.na(threshold) & is.na(negativeThreshold)) {
+    stop("Specify a numeric value for 'threshold' and/or 'negativeThreshold'.",
          call. = FALSE)
   }
+  if ((!is.na(threshold) &
+       !is.numeric(threshold)) |
+      (!is.na(negativeThreshold) &
+       !is.numeric(negativeThreshold))) {
+    stop("Specify a numeric value for 'threshold' and/or 'negativeThreshold'.",
+         call. = FALSE)
+  }
+  
+  if(!is.na(negativeThreshold) & ignore_sign == TRUE){
+    stop("Do not specify a negative threshold together with option 'ignore_sign' set to TRUE.")
+  }
+  
+  if (is.na(threshold)) {
+    threshold <- Inf
+  }
+  if (is.na(negativeThreshold)) {
+    negativeThreshold <- -Inf
+  }
+  
+  if (threshold <= 0) {
+    stop("Specify a positive number for 'threshold'.", call. = FALSE)
+  }
+  if (negativeThreshold >= 0) {
+    stop("Specify a negative number for 'negativeThreshold'.", call. = FALSE)
+  }
+  
   # need integer numbers, otherwise "%%" causes trouble with i386
   window_size <- as.integer(window_size)
   overlap <- as.integer(overlap)
@@ -80,8 +144,8 @@ calc_candidate_regions <- function(scan,
     stop("'overlap' has to be zero or an integer factor of 'window_size'.",
          call. = FALSE)
   }
-  if (min_n_mrk < 1 | min_n_mrk %% 1 != 0L) {
-    stop("'min_n_mrk' has to be a positive integer number.", call. = FALSE)
+  if (min_n_mrk < 0 | min_n_mrk %% 1 != 0L) {
+    stop("'min_n_mrk' has to be a non-negative integer number.", call. = FALSE)
   }
   if (min_n_extr_mrk < 0 | min_n_extr_mrk %% 1L != 0L) {
     stop("'min_n_extr_mrk' has to be a positive integer number.",
@@ -109,7 +173,8 @@ calc_candidate_regions <- function(scan,
   
   # perform calculation
   ## remove NA
-  scan <- scan[!is.na(scan[[score_column_nr]]), ]
+  if (!keepNA)
+    scan <- scan[!is.na(scan[[score_column_nr]]), ]
   
   if (ignore_sign) {
     score <- abs(scan[[score_column_nr]])
@@ -139,6 +204,13 @@ calc_candidate_regions <- function(scan,
       chr_pos <- scan[scan$CHR == chr, "POSITION"]
       chr_score <- score[scan$CHR == chr]
       
+      if (threshold == Inf) {
+        chr_score[chr_score > 0] <- 0
+      }
+      if (negativeThreshold == -Inf) {
+        chr_score[chr_score < 0] <- 0
+      }
+      
       chr_cand_reg <- data.frame(matrix(ncol = 6, nrow = 0))
       
       if (overlap != 0) {
@@ -149,44 +221,51 @@ calc_candidate_regions <- function(scan,
       
       for (offset in offsets) {
         breaks <- seq(
-          floor(min(chr_pos) / window_size) * window_size + offset,
-          max(chr_pos) + window_size - 1,
+          floor(min(chr_pos - right) / window_size) * window_size + offset,
+          max(chr_pos+!right) + window_size - 1,
           window_size
         )
+        
         # empty window (may arise if only a single value in chr_pos)
         if (length(breaks) < 2)
           next
         
-        windows <- cut(chr_pos,
-                       breaks = breaks,
-                       right = right,
-                       labels = FALSE)
+        windows <- cut(
+          chr_pos,
+          breaks = breaks,
+          right = right,
+          labels = 1:(length(breaks) - 1)
+        )
         
-        n_mrk <- tapply(chr_score, windows, function(x) {
-          length(x)
-        })
+        n_mrk <- tapply(chr_score, windows, length, default = 0L)
         
         mean_mrk <- tapply(chr_score, windows, function(x) {
-          mean(x)
+          non_zero <- x[!is.na(x) & x != 0]
+          ifelse(length(non_zero) == 0, NA, mean(abs(non_zero)))
         })
         
         max_mrk <- tapply(chr_score, windows, function(x) {
-          max(x)
+          non_zero <- x[!is.na(x) & x != 0]
+          ifelse(length(non_zero) == 0, NA, max(abs(non_zero)))
         })
         
         n_extr_mrk <- tapply(chr_score, windows, function(x) {
-          sum(x >= threshold)
+          sum(x >= threshold | x <= negativeThreshold, na.rm = TRUE)
         })
         
         mean_extr_mrk <- tapply(chr_score, windows, function(x) {
-          mean(x[x >= threshold])
+          extreme_values <- x[!is.na(x) &
+                                (x >= threshold |
+                                   x <= negativeThreshold)]
+          ifelse(length(extreme_values) == 0, NA, mean(abs(extreme_values)))
         })
         
-        perc_extr_mrk <- n_extr_mrk / n_mrk * 100
+        perc_extr_mrk <- ifelse(n_mrk == 0 , NA, n_extr_mrk / n_mrk * 100)
         
         condition <- n_mrk >= min_n_mrk &
-          n_extr_mrk >= min_n_extr_mrk &
-          perc_extr_mrk >= min_perc_extr_mrk
+          (is.na(n_extr_mrk) | (n_extr_mrk >= min_n_extr_mrk)) &
+          (is.na(perc_extr_mrk) |
+             (perc_extr_mrk >= min_perc_extr_mrk))
         
         # subset all vectors by condition
         n_mrk <- n_mrk[condition]
@@ -196,7 +275,7 @@ calc_candidate_regions <- function(scan,
         perc_extr_mrk <- perc_extr_mrk[condition]
         mean_extr_mrk <- mean_extr_mrk[condition]
         
-        cand_window_nr <- as.integer(names(n_extr_mrk))
+        cand_window_nr <- which(condition)
         
         if (length(cand_window_nr) > 0) {
           # append chromosomal candidates for each window offset
@@ -244,12 +323,14 @@ calc_candidate_regions <- function(scan,
               chr_cand_reg[i, 6] <-
                 max(chr_score[index])
               chr_cand_reg[i, 7] <-
-                sum(chr_score[index] >= threshold)
+                sum(chr_score[index] >= threshold |
+                      chr_score[index] <= negativeThreshold)
               chr_cand_reg[i, 8] <- round(chr_cand_reg[i, 7] /
                                             chr_cand_reg[i, 4] * 100, 2)
               chr_cand_reg[i, 9] <-
-                mean(chr_score[index][chr_score[index] >=
-                                        threshold])
+                mean(abs(chr_score[index][chr_score[index] >=
+                                            threshold |
+                                            chr_score[index] <= negativeThreshold]))
               #eliminate all right neighbors
               chr_cand_reg <- chr_cand_reg[-((i + 1):(i +
                                                         j)), ]
@@ -267,13 +348,9 @@ calc_candidate_regions <- function(scan,
   } else{
     # window_size = 1 => report subset of scan with individual extremal markers
     cand_reg <-
-      scan[score >= threshold,
-           c("CHR", "POSITION", "POSITION", colnames(scan)[score_column_nr])]
+      scan[score >= threshold, c("CHR", "POSITION", "POSITION", colnames(scan)[score_column_nr])]
     
-    colnames(cand_reg) <- c("CHR",
-                            "START",
-                            "END",
-                            "EXTR_MRK")
+    colnames(cand_reg) <- c("CHR", "START", "END", "EXTR_MRK")
   }
   
   ## output
