@@ -20,7 +20,8 @@ SEXP CALL_FURCATION(SEXP data_, SEXP nbr_chr_, SEXP foc_mrk_, SEXP end_mrk_, SEX
 	SEXP node_parent_ = PROTECT(allocVector(INTSXP, 2 * nbr_chr - 1));
 	SEXP node_with_missing_data_ = PROTECT(allocVector(INTSXP, 2 * nbr_chr - 1));
 	SEXP label_parent_ = PROTECT(allocVector(INTSXP, nbr_chr));
-
+  SEXP nbr_nodes_ = PROTECT(allocVector(INTSXP, 1));
+		
 	//get C references for R vectors
 	int* node_mrk = INTEGER(node_mrk_);
 	int* node_parent = INTEGER(node_parent_);
@@ -32,22 +33,20 @@ SEXP CALL_FURCATION(SEXP data_, SEXP nbr_chr_, SEXP foc_mrk_, SEXP end_mrk_, SEX
 	calc_furcation(data, nbr_chr, foc_mrk, end_mrk, foc_all, lim_haplo, phased, node_mrk, node_parent,
 			node_with_missing_data, &nbr_nodes, label_parent);
 
-	//reset length from maximal length to actual length
-	SETLENGTH(node_mrk_, nbr_nodes);
-	SETLENGTH(node_parent_, nbr_nodes);
-	SETLENGTH(node_with_missing_data_, nbr_nodes);
-
+	INTEGER(nbr_nodes_)[0] = nbr_nodes;
+	
 	//create R list of length 8
-	SEXP list_ = PROTECT(allocVector(VECSXP, 4));
+	SEXP list_ = PROTECT(allocVector(VECSXP, 5));
 
 	//add R vectors to list
 	SET_VECTOR_ELT(list_, 0, node_mrk_);
 	SET_VECTOR_ELT(list_, 1, node_parent_);
 	SET_VECTOR_ELT(list_, 2, node_with_missing_data_);
 	SET_VECTOR_ELT(list_, 3, label_parent_);
+	SET_VECTOR_ELT(list_, 4, nbr_nodes_);
 
 	//unprotect all created R objects
-	UNPROTECT(5);
+	UNPROTECT(6);
 
 	//return R list
 	return list_;
