@@ -81,7 +81,7 @@ test_that("checked_calc_candidate_regions", {
                                         cr.ihs.cgu,
                                         threshold = 4,
                                         pval = TRUE)
-  expect_identical(cr.ihs.cgu, region_stats.cgu)
+  expect_identical(cr.ihs.cgu, region_stats.cgu, attrib.as.set = TRUE)
   
   sink()
   file.remove("test_calc_candidate_regions.log")
