@@ -60,6 +60,19 @@ test_that("checked_scan_hh_full", {
   INES <- IES * nhap(hh) / (scan2$NHAPLO_A + scan2$NHAPLO_D)
   expect_equal(scan2$INES, INES)
   
+  #compare haplen and pairwise haplen
+  f <- calc_furcation(hh, mrk = "F1205400")
+  h <- calc_haplen(f)
+  #all pairwise shared haplotype lengths to the left of the focal marker
+  m_left <- calc_pairwise_haplen(hh, mrk = "F1205400", side = "left")
+  #get for each chromosome the maximum shared haplotype length
+  max_left <- apply(m_left, 1, max)
+  expect_equal(max_left, h$position - h$haplen$MIN, check.attributes = FALSE)
+  #the same for shared haplotypes to the right of the focal marker
+  m_right <- calc_pairwise_haplen(hh, mrk = "F1205400", side = "right")
+  max_right <- apply(m_right, 1, max)
+  expect_equal(max_right, h$haplen$MAX - h$position, check.attributes = FALSE)
+  
   #test max_extend and maxgap
   hh <- data2haplohh(
     hap_file = "example1.hap",
